package com.jiuqi.grid;

import com.jiuqi.util.Stream;
import com.jiuqi.util.StreamException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

/* loaded from: com_jiuqi_grid.jar:com/jiuqi/grid/GridFieldList.class */
public final class GridFieldList implements Serializable {
    private List list;
    private GridData parent;
    private boolean autoMerge;
    private com.jiuqi.util.Matrix rectMaps;
    private Set tmpRects;

    public GridFieldList(GridData aParent) {
        this();
        this.parent = aParent;
    }

    public GridFieldList() {
        Comparator rectCmp = new Comparator() { // from class: com.jiuqi.grid.GridFieldList.1
            @Override // java.util.Comparator
            public int compare(Object o1, Object o2) {
                CellField f1 = (CellField) o1;
                CellField f2 = (CellField) o2;
                return f1.compareTo(f2);
            }
        };
        this.list = new ArrayList();
        this.rectMaps = new com.jiuqi.util.Matrix();
        this.tmpRects = new TreeSet(rectCmp);
    }

    public int count() {
        return this.list.size();
    }

    public GridData getParent() {
        return this.parent;
    }

    public CellField get(int index) {
        return (CellField) this.list.get(index);
    }

    public void remove(int index) {
        CellField field = (CellField) this.list.remove(index);
        for (int col = field.left; col <= field.right; col++) {
            for (int row = field.top; row <= field.bottom; row++) {
                this.rectMaps.setItem(col, row, (Object) null);
            }
        }
    }

    protected boolean remove(CellField field) {
        for (int i = 0; i < this.list.size(); i++) {
            if (field == this.list.get(i)) {
                remove(i);
                return true;
            }
        }
        return false;
    }

    public CellField getMergeRect(int col, int row) {
        if (col >= this.rectMaps.getColCount() || row >= this.rectMaps.getRowCount()) {
            return null;
        }
        return (CellField) this.rectMaps.getItem(col, row);
    }

    public boolean checkCanMerge(CellField field) {
        for (int col = field.left; col <= field.right; col++) {
            for (int row = field.top; row <= field.bottom; row++) {
                CellField f = getMergeRect(col, row);
                if (f != null) {
                    switch (CellField.compareField(f, field)) {
                        case CellField.FIELDS_CROSS /* -2 */:
                        case 1:
                            return false;
                    }
                }
            }
        }
        return true;
    }

    public boolean addMergeRect(CellField field) {
        if (checkCanMerge(field)) {
            this.tmpRects.clear();
            if (this.rectMaps.getColCount() <= field.right) {
                this.rectMaps.setColCount(field.right + 1);
            }
            if (this.rectMaps.getRowCount() <= field.bottom) {
                this.rectMaps.setRowCount(field.bottom + 1);
            }
            for (int col = field.left; col <= field.right; col++) {
                for (int row = field.top; row <= field.bottom; row++) {
                    CellField f = (CellField) this.rectMaps.getItem(col, row);
                    if (f != null && !this.tmpRects.contains(f)) {
                        this.tmpRects.add(f);
                    }
                    this.rectMaps.setItem(col, row, field);
                }
            }
            if (!this.tmpRects.isEmpty()) {
                this.list.removeAll(this.tmpRects);
                this.tmpRects.clear();
            }
            this.list.add(field);
            return true;
        }
        return false;
    }

    public int removeMergeRect(CellField field) {
        int count = 0;
        for (int col = field.left; col <= field.right; col++) {
            for (int row = field.top; row <= field.bottom; row++) {
                CellField f = getMergeRect(col, row);
                if (f != null) {
                    if (field.containField(f)) {
                        remove(f);
                        count++;
                    } else {
                        GridCell originCell = this.parent.getCellEx(f.left, f.top);
                        if (f.left <= field.right && f.right > field.right) {
                            f.left = field.right + 1;
                        }
                        if (f.top <= field.bottom && f.bottom > field.bottom) {
                            f.top = field.bottom + 1;
                        }
                        GridCell currentCell = this.parent.getCellEx(f.left, f.top);
                        if (originCell != null && currentCell != null && !originCell.equals(currentCell)) {
                            currentCell.setBackColor(originCell.getBackColor());
                            currentCell.setBackStyle(originCell.getBackStyle());
                            currentCell.setLEdgeStyle(originCell.getLEdgeStyle());
                            currentCell.setLEdgeColor(originCell.getLEdgeColor());
                            currentCell.setTEdgeStyle(originCell.getTEdgeStyle());
                            currentCell.setTEdgeColor(originCell.getTEdgeColor());
                            currentCell.setREdgeStyle(originCell.getREdgeStyle());
                            currentCell.setREdgeColor(originCell.getREdgeColor());
                            currentCell.setBEdgeStyle(originCell.getBEdgeStyle());
                            currentCell.setBEdgeColor(originCell.getBEdgeColor());
                            currentCell.setShowText("");
                            this.parent.setCell(currentCell);
                        }
                    }
                }
            }
        }
        return count;
    }

    public void delRows(int startRow, int count) {
        int bottom = (startRow + count) - 1;
        for (int i = count() - 1; i >= 0; i--) {
            CellField f = get(i);
            int t = f.top < startRow ? startRow : f.top;
            int b = f.bottom < bottom ? f.bottom : bottom;
            if (b >= t) {
                f.bottom -= (b - t) + 1;
                if (f.bottom < f.top) {
                    remove(i);
                }
            } else if (f.top > bottom) {
                f.top -= count;
                f.bottom -= count;
            }
        }
        if (startRow < this.rectMaps.getRowCount()) {
            this.rectMaps.deleteRow(startRow, count);
        }
    }

    public void delCols(int startCol, int count) {
        int right = (startCol + count) - 1;
        for (int i = count() - 1; i >= 0; i--) {
            CellField f = get(i);
            int l = f.left < startCol ? startCol : f.left;
            int r = f.right < right ? f.right : right;
            if (r >= l) {
                f.right -= (r - l) + 1;
            }
            if (f.right < f.left) {
                remove(i);
            } else if (f.left > right) {
                f.left -= count;
                f.right -= count;
            }
        }
        if (startCol < this.rectMaps.getColCount()) {
            this.rectMaps.deleteCol(startCol, count);
        }
    }

    public void insertRows(int startRow, int count) {
        Object[] colItems = (Object[]) null;
        List adjRects = new ArrayList();
        for (int i = 0; i < count(); i++) {
            CellField f = get(i);
            if (startRow > f.top && startRow <= f.bottom) {
                f.bottom += count;
                if (colItems == null) {
                    colItems = new Object[this.rectMaps.getColCount()];
                }
                for (int col = f.left; col <= f.right; col++) {
                    colItems[col] = f;
                }
            } else if (startRow <= f.top) {
                f.top += count;
                f.bottom += count;
            } else if (startRow == f.bottom + 1 && f.right > f.left && this.autoMerge) {
                adjRects.add(f);
            }
        }
        if (this.rectMaps.getRowCount() > 0 && startRow <= this.rectMaps.getRowCount()) {
            this.rectMaps.insertRow(startRow, count, colItems);
        }
        for (int i2 = 0; i2 < adjRects.size(); i2++) {
            CellField f2 = (CellField) adjRects.get(i2);
            for (int row = startRow; row < startRow + count; row++) {
                addMergeRect(new CellField(f2.left, row, f2.right, row));
            }
        }
    }

    public void insertCols(int startCol, int count) {
        Object[] rowItems = (Object[]) null;
        List adjRects = new ArrayList();
        for (int i = 0; i < count(); i++) {
            CellField f = get(i);
            if (startCol > f.left && startCol <= f.right) {
                f.right += count;
                if (rowItems == null) {
                    rowItems = new Object[this.rectMaps.getRowCount()];
                }
                for (int row = f.top; row <= f.bottom; row++) {
                    rowItems[row] = f;
                }
            } else if (startCol <= f.left) {
                f.left += count;
                f.right += count;
            } else if (startCol == f.right + 1 && f.top < f.bottom && this.autoMerge) {
                adjRects.add(f);
            }
        }
        if (this.rectMaps.getColCount() > 0 && startCol <= this.rectMaps.getColCount()) {
            this.rectMaps.insertCol(startCol, count, rowItems);
        }
        for (int i2 = 0; i2 < adjRects.size(); i2++) {
            CellField f2 = (CellField) adjRects.get(i2);
            for (int col = startCol; col < startCol + count; col++) {
                addMergeRect(new CellField(col, f2.top, col, f2.bottom));
            }
        }
    }

    public void extendRows(int startRow, int count) {
        for (int i = 0; i < count(); i++) {
            CellField f = get(i);
            if (startRow >= f.top && startRow <= f.bottom) {
                f.bottom += count;
            } else if (startRow < f.top) {
                f.top += count;
                f.bottom += count;
            }
        }
        if (this.rectMaps.getRowCount() > 0 && startRow < this.rectMaps.getRowCount()) {
            this.rectMaps.duplicateRow(startRow, count);
        }
    }

    public void extendCols(int startCol, int count) {
        for (int i = 0; i < count(); i++) {
            CellField f = get(i);
            if (startCol >= f.left && startCol <= f.right) {
                f.right += count;
            } else if (startCol < f.left) {
                f.left += count;
                f.right += count;
            }
        }
        if (this.rectMaps.getColCount() > 0 && startCol < this.rectMaps.getColCount()) {
            this.rectMaps.duplicateCol(startCol, count);
        }
    }

    public List findFieldsInCols(int from, int to) {
        List result = new ArrayList();
        for (int i = 0; i < count(); i++) {
            CellField f = get(i);
            if (f.left >= from && f.right <= to) {
                result.add(f);
            }
        }
        return result;
    }

    public List findFieldsInRows(int from, int to) {
        List result = new ArrayList();
        for (int i = 0; i < count(); i++) {
            CellField f = get(i);
            if (f.top >= from && f.bottom <= to) {
                result.add(f);
            }
        }
        return result;
    }

    public void loadFromStream(Stream stream, int size) throws StreamException {
        this.list.clear();
        this.rectMaps.clear();
        for (int i = 0; i < size / 16; i++) {
            CellField cf = new CellField();
            cf.left = stream.readInt();
            cf.top = stream.readInt();
            cf.right = stream.readInt();
            cf.bottom = stream.readInt();
            addMergeRect(cf);
        }
    }

    public void saveToStream(Stream stream) throws StreamException {
        for (int i = 0; i < this.list.size(); i++) {
            CellField cf = (CellField) this.list.get(i);
            stream.writeInt(cf.left);
            stream.writeInt(cf.top);
            stream.writeInt(cf.right);
            stream.writeInt(cf.bottom);
        }
    }

    public boolean getAutoMerge() {
        return this.autoMerge;
    }

    public void setAutoMerge(boolean value) {
        this.autoMerge = value;
    }
}
